﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Webus.Analysis;
using System.Text.RegularExpressions;

namespace CodeSearch
{
    class CodeAnalyzer : IAnalyzer
    {
        public ITokenStream GetTokenStream(string value)
        {
            return new CodeTokenStream(value);
        }

        public ITokenStream GetTokenStream(Webus.Documents.Field field)
        {
            if (field.Name == "Code")
            {
                return this.GetTokenStream(field.Value.ToString());
            }
            else
            {
                return null;
            }
        }
    }

    class CodeTokenStream : ITokenStream
    {
        HashSet<string> stops = new HashSet<string>(new string[] { 
            "abstract",
            "event",	
            "new",	
            "struct",
            "as",	
            "explicit",	
            "null",	
            "switch",
            "base",	
            "extern",	
            "object",	
            "this",
            "bool",	
            "false",	
            "operator",	
            "throw",
            "break",	
            "finally",	
            "out",	
            "true",
            "byte",	
            "fixed",	
            "override",	
            "try",
            "case",	
            "float",	
            "params",	
            "typeof",
            "catch",	
            "for",	
            "private",	
            "uint",
            "char",	
            "foreach",	
            "protected",	
            "ulong",
            "checked",	
            "goto",	
            "public",	
            "unchecked",
            "class",	
            "if",	
            "readonly",	
            "unsafe",
            "const",	
            "implicit",	
            "ref",	
            "ushort",
            "continue",	
            "in",	
            "return",	
            "using",
            "decimal",	
            "int",	
            "sbyte",	
            "virtual",
            "default",	
            "interface",	
            "sealed",	
            "volatile",
            "delegate",	
            "internal",	
            "short",	
            "void",
            "do",	
            "is",	
            "sizeof",	
            "while",
            "double",	
            "lock",	
            "stackalloc",	 
            "else",	
            "long",	
            "static",	 
            "enum",	
            "namespace",	
            "string"
        });
        Queue<Token> m_Buffer = new Queue<Token>();
        public CodeTokenStream(string text)
        {
            MatchCollection mc = Regex.Matches(text, @"\w+");
            foreach (Match m in mc)
            {
                var key = m.Value.ToLower();
                if (key.Length <= 50 && stops.Contains(key) == false)
                {
                    m_Buffer.Enqueue(new Token(key, m.Index, m.Length));
                }
            }
        }
        public void Close()
        {
            this.Dispose();
        }

        public Token Current
        {
            get;
            private set;
        }

        public bool Next()
        {
            if (m_Buffer.Count > 0)
            {
                this.Current = m_Buffer.Dequeue();
                return true;
            }
            this.Current = null;
            return false;
        }

        public void Dispose()
        {
            m_Buffer = null;
        }
    }
}
